import {StoreComponent} from "../../../../../app/storage/StoreComponent";
import {Character} from "../../game-model/characters/Character";
import {SimpleTemplateGamesystem} from "../../game-model/templates/simpleGamesystem/SimpleTemplateGamesystem";
import {ScriptAccountActionParser} from "../gamesystemParser/ScriptAccountActionParser";
import {ScriptAccountConditionParser} from "../gamesystemParser/ScriptAccountConditionParser";
import {ScriptAccount} from "../../game-model/scriptAccounts/ScriptAccount";
import {SimpleTemplateState} from "../../game-model/templates/simpleGamesystem/SimpleTemplateState";
import {SimpleTemplateTransition} from "../../game-model/templates/simpleGamesystem/SimpleTemplateTransition";
import {CharacterRelation} from "../../game-model/characters/CharacterRelation";
import {InventorySlotParser} from "./InventorySlotParser";
import {ItemGroupCharacteristic} from "../../game-model/inventory/ItemgroupCharacteristic";
import {ItemGroup} from "../../game-model/inventory/ItemGroup";


export class CharacterParser {

  characterSpecificGamesystems: SimpleTemplateGamesystem[]
  characterRelationSpecificGamesystems: SimpleTemplateGamesystem[]
  scriptAccountConditionParser: ScriptAccountConditionParser
  scriptAccountActionParser: ScriptAccountActionParser
  inventorySlotParser: InventorySlotParser

  constructor(characterSpecificGamesystems: SimpleTemplateGamesystem[], characterRelationSpecificGamesystems: SimpleTemplateGamesystem[], scriptAccounts: ScriptAccount[], characteristics: ItemGroupCharacteristic[], itemgroups: ItemGroup[]) {
    this.characterSpecificGamesystems = characterSpecificGamesystems;
    this.characterRelationSpecificGamesystems = characterRelationSpecificGamesystems;
    this.scriptAccountActionParser = new ScriptAccountActionParser(scriptAccounts);
    this.scriptAccountConditionParser = new ScriptAccountConditionParser(scriptAccounts)
    this.inventorySlotParser = new InventorySlotParser(characteristics, itemgroups);
  }


  public parseCharacters(characters: StoreComponent[]): Character[] {
    const loadedCharacters: Character[] = []
    characters.forEach(character => loadedCharacters.push(this.parseSingleCharacter(JSON.parse(character.jsonString), loadedCharacters)))
    return loadedCharacters;
  }

  private parseSingleCharacter(characterData: any, loadedCharacters: Character[]): Character {
    const character = new Character(characterData.componentName, characterData.componentDescription);

    loadedCharacters.forEach(loadedCharacter => {
      const characterRelation = new CharacterRelation(loadedCharacter, character);
      loadedCharacter.addCharacterRelation(characterRelation)
      character.addCharacterRelation(characterRelation)
    })

    const templateSpecificGamesystems = this.parseCharacterSpecificGamesystems(character, characterData.characterSpecificTemplateSystems, this.characterSpecificGamesystems);
    templateSpecificGamesystems.forEach(system => character.addCharacterSpecificSimpleTemplatesystem(system))

    const characterRelationGamesystems = this.parseCharacterSpecificGamesystems(character, characterData.characterRelationGamesystems, this.characterRelationSpecificGamesystems)
    characterRelationGamesystems.forEach(gamesystem => character.addAsymetricCharacterRelationGamesystem(gamesystem))

    character.inventorySlots = this.inventorySlotParser.parseInventorySlots(characterData.inventorySlots);
    return character;

  }

  private parseCharacterSpecificGamesystems(character: Character, characterSpecificGamesystems: any, templateGamesystems: SimpleTemplateGamesystem[]): SimpleTemplateGamesystem[] {
    const result: SimpleTemplateGamesystem[] = []
    for(let i=0; i<characterSpecificGamesystems.length; i++) {
      const characterSpecificGamesystem = characterSpecificGamesystems[i];
      result.push(this.parseSingleCharacterSpecificGamesystem(character, characterSpecificGamesystem, templateGamesystems)!)
    }
    return result;
  }

  private parseSingleCharacterSpecificGamesystem(character: Character, characterSpecificGamesystem: any, templateGamesystems: SimpleTemplateGamesystem[]): SimpleTemplateGamesystem | undefined{
    const referencedGamesystem = this.findCharacterSpecificGamesystem(templateGamesystems, characterSpecificGamesystem.componentName)

    if(referencedGamesystem != undefined) {
      for(let i=0; i<characterSpecificGamesystem.states.length; i++) {
        const stateReference = characterSpecificGamesystem.states[i];
        const state = this.findReferencedState(referencedGamesystem, stateReference.stateLabel)! as SimpleTemplateState
        const conditions = this.scriptAccountConditionParser.parseStoredConditions(stateReference.conditionMap);

        state.conditionMap.set(character, conditions)
        state.initialMap.set(character, stateReference.initialMap)
      }

      for(let i=0; i<characterSpecificGamesystem.transitions.length; i++) {
        const transitionReference = characterSpecificGamesystem.transitions[i];

        const transition = this.findReferencedTransition(referencedGamesystem, transitionReference.startingState, transitionReference.endingState) as SimpleTemplateTransition;
        const condititions = this.scriptAccountConditionParser.parseStoredConditions(transitionReference.conditionMap);
        const actions = this.scriptAccountActionParser.parseActions(transitionReference.actionMap);

        transition!.actionMap.set(character, actions);
        transition!.conditionMap.set(character, condititions);
      }
    }

    return referencedGamesystem;
  }
  private findCharacterSpecificGamesystem(gamesystems: SimpleTemplateGamesystem[], componentName: string): SimpleTemplateGamesystem | undefined{
    return gamesystems.find(gamesystem => gamesystem.componentName === componentName)
  }

  private findReferencedState(gamesystem: SimpleTemplateGamesystem, stateLabel: string) {
    return gamesystem.states.find(state => state.stateLabel === stateLabel);
  }

  private findReferencedTransition(gamesystem: SimpleTemplateGamesystem, startingLabel: string, endingLabel: string) {
    return gamesystem.transitions.find(transition => transition.startingState.stateLabel === startingLabel && transition.endingState.stateLabel === endingLabel);
  }

}
