import {ItemGroup} from "../../game-model/inventory/ItemGroup";
import {StoreComponent} from "../../../../../app/storage/StoreComponent";
import {Item} from "../../game-model/inventory/Item";
import {ModelComponentType} from "../../game-model/ModelComponentType";
import {ConcreteItemGroup} from "../../game-model/inventory/ConcreteItemGroup";
import {ItemgroupCharacteristicValue} from "../../game-model/inventory/ItemgroupCharacteristicValue";

export class ItemParser {
  parsedItemgroups: ItemGroup[] = []

  constructor(itemgroups: ItemGroup[] = []) {
    this.parsedItemgroups = itemgroups;
  }

  parseItems(storeComponents: StoreComponent[]): Item[] {
    const parsedItems: Item[] = []
    storeComponents.forEach(storeComponent => parsedItems.push(this.parseSingleItem(storeComponent)));
    return parsedItems;
  }

  private parseSingleItem(storeComponent: StoreComponent) {
    const parsedData = JSON.parse(storeComponent.jsonString);

    const componentName = parsedData.componentName;
    const componentDescription = parsedData.componentDescription;

    const manuallyInheritedGroups = this.parseInheritedGroups(parsedData.manuallyInheritedGroups);
    const hierarchyInheritedGroups = this.parseInheritedGroups(parsedData.hierarchyInheritedGroups);

    const item = new Item(componentName, componentDescription, ModelComponentType.ITEM);
    item.manuallyInheritedGroups = manuallyInheritedGroups;
    item.hierarchyInheritedGroups = hierarchyInheritedGroups;

    const owningGroup = hierarchyInheritedGroups.find(itemgroup => itemgroup instanceof ConcreteItemGroup);
    if (owningGroup) {
      (owningGroup as ConcreteItemGroup).items.push(item);
    } else {
      console.log("[ERROR] No group found for item: ", item.componentName)
    }


    item.itemCharacteristicValues = this.parseItemCharacteristicValues(parsedData.itemCharacteristicValues);
    return item;
  }

  private parseInheritedGroups(inheritedGroupsData: any) {
    const inheritedGroups: ItemGroup[] = []
    for(let i=0; i<inheritedGroupsData.length; i++) {
      const inheritedGroupName = inheritedGroupsData[i];

      const matchedGroup = this.parsedItemgroups.find(group => group.componentName === inheritedGroupName);
      if (matchedGroup) {
        inheritedGroups.push(matchedGroup);
      }
    }

    return inheritedGroups;
  }

  private parseItemCharacteristicValues(data: any): ItemgroupCharacteristicValue[] {
    const values: ItemgroupCharacteristicValue[] = []
    for(let i=0; i<data.length; i++) {
      const key = this.findItemCharacteristic(data[i].key);
      if (key) {
        const characteristicValue = new ItemgroupCharacteristicValue(key!, data[i].value)
        values.push(characteristicValue)
      } else {
        console.log("[ERROR] Characteristic not found: ", data[i].key)
      }
    }
    return values;
  }

  private findItemCharacteristic(itemCharacteristicName: string) {
    for(let i=0; i<this.parsedItemgroups.length; i++) {
      const characteristic = this.parsedItemgroups[i].itemGroupCharacteristics
        .find(characteristic=> characteristic.characteristicName === itemCharacteristicName);

      if(characteristic != undefined) {
        return characteristic;
      }
    }
    return undefined;
  }


}
